import ij.*;
import ij.io.*;
import ij.process.*;
import ij.gui.*;
import java.io.*;
import java.nio.file.*;
import java.awt.*;
import ij.plugin.*;
import ij.plugin.frame.*;

    /**
      This plugin copies a single directory of DICOM datasets into a hierarchical structure of Study/Series/Images.

      @author Fred Damen <fred@damen.org>

      Version History:
      2019-04-01: Created
    */

public class DICOM_explode implements PlugIn {

   public void run(String arg) {
      File fdir = null;
      File tdir = null; 
      {
         DirectoryChooser dc = new DirectoryChooser("Directory of your combined DICOM datasets");
         String sdir = dc.getDirectory();
         if (sdir == null) return;
         fdir = new File(sdir);
      }

      {
         DirectoryChooser dc = new DirectoryChooser("Parent Directory of your separaed DICOM datasets");
         String sdir = dc.getDirectory();
         if (sdir == null) return;
         tdir = new File(sdir);
      }

      int i=1;
      File[] files = fdir.listFiles();
      for(File f : files) {
         IJ.showStatus("Exploding DICOM files...");
         IJ.showProgress(((float)i++)/files.length);
         String exam   = "";
         String series = "";
         String image  = "";
         ImagePlus imp = (ImagePlus)IJ.runPlugIn("ij.plugin.DICOM", f.getAbsolutePath());
         for(String line : imp.getStack().getSliceLabel(1).split("\n",2)[1].split("\n")) {
            String[] kv = line.split(" ",2);
                 if (kv[0].equals("0020,0010"))   exam   = kv[1].split(": ",2)[1].trim();
            else if (kv[0].equals("0020,0011"))   series = kv[1].split(": ",2)[1].trim();
            else if (kv[0].equals("0020,0013")) { image  = kv[1].split(": ",2)[1].trim(); break; }
            }
         File t = new File(new File(new File(tdir,"EX"+exam),"SE"+series),"IM"+image+".dcm");
         t.getParentFile().mkdirs();
         try { Files.copy(f.toPath(),t.toPath()); }
         catch(IOException ioe) { IJ.log(ioe.toString()); }
         }
      IJ.showStatus("Poof");
      IJ.showProgress(1.0);
   }
}
